/*
 *  Dr.Mario port for Uzebox
 *  Copyright (C) 2009  Codecrank
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Uzebox is a reserved trade mark
*/

#include <avr/io.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdio.h>
#include <avr/pgmspace.h>
#include <uzebox.h>

#include "data/fonts.pic.inc"
#include "data/patches.inc"

#include "drmario_main.c"
#include "drmario_sprites.c"
#include "drmario_music.inc"
#include "drmario_title.c"
//#include "pilldropper.inc"
//#include "md.inc"


#define DROPPER_ID 3

#define MOVE_LEFT 1
#define MOVE_RIGHT 2
#define MOVE_DOWN 3
#define MOVE_SPIN_C 4
#define MOVE_SPIN_C_C 6
#define MOVE_START 5

#define PLAYER1 0
#define PLAYER2 1

#define BIT_9 256
#define BIT_8 128
#define BIT_7 64
#define BIT_6 32
#define BIT_5 16
#define BIT_4 8
#define BIT_3 4
#define BIT_2 2
#define BIT_1 1
#define CURSOR_TILE 62

#define BOTTLE_FIRST_COLUMN 4
#define BOTTLE2_FIRST_COLUMN 28
#define BOTTLE_LAST_COLUMN 11
#define BOTTLE_FIRST_ROW 8
#define BOTTLE_LAST_ROW 24
#define BOTTLE_WIDTH 8
#define BOTTLE_DEPTH 17
#define VIRUS_LEFT_ROW  23
#define VIRUS_LEFT_COLUMN1 19
#define VIRUS_LEFT_COLUMN2 22

/*
1 - 11 = red
12 - 22 = blue
23 - 33 = yellow
*/


#define LEFT_RED 1
#define RIGHT_RED 2
#define BOTTOM_RED 3
#define TOP_RED 4
#define SINGLE_RED 5
#define VIRUS_RED1 6
#define VIRUS_RED2 7
#define DISSAPEAR_ANIM_1_R 8
#define DISSAPEAR_ANIM_2_R 9
#define DISSAPEAR_ANIM_3_R 10
#define DISSAPEAR_ANIM_4_R 11

#define LEFT_BLUE 12
#define RIGHT_BLUE 13
#define BOTTOM_BLUE 14
#define TOP_BLUE 15
#define SINGLE_BLUE 16
#define VIRUS_BLUE1 	17
#define VIRUS_BLUE2 	18
#define DISSAPEAR_ANIM_1_B 19
#define DISSAPEAR_ANIM_2_B 20
#define DISSAPEAR_ANIM_3_B 21
#define DISSAPEAR_ANIM_4_B 22

#define LEFT_YELLOW 23
#define RIGHT_YELLOW 24
#define BOTTOM_YELLOW 25
#define TOP_YELLOW 26
#define SINGLE_YELLOW 27
#define VIRUS_YELLOW1 	28
#define VIRUS_YELLOW2 	29
#define DISSAPEAR_ANIM_1_Y 30
#define DISSAPEAR_ANIM_2_Y 31
#define DISSAPEAR_ANIM_3_Y 32
#define DISSAPEAR_ANIM_4_Y 33
 
#define CURSOR1		54
#define CURSOR2		55	

#define HORIZ 0
#define VERT 1

#define FULL 2
#define HALF 1


const char strGame[] PROGMEM = "GAME";
const char strOver[] PROGMEM = "OVER";

const char strS[] PROGMEM = "S";
const char strM[] PROGMEM = "M";

const char strPress[] PROGMEM = "PRESS";
const char strStart[] PROGMEM = "START";
const char strBlank[] PROGMEM = "     ";

const char strBesttime[] PROGMEM = "BEST TIME";
const char strBugs[] PROGMEM = "BUGS";

const char strBackToMenu[] PROGMEM = "BACK TO MENU";

const char strResetHighScore[] PROGMEM =  "HOLD SERVICE BUTTON THROUGH";
const char strResetHighScoreTwo[] PROGMEM="UZEBOX COIN TO RESET HI SCORE";
const char strCoinsPerCreditOne[] PROGMEM = "1P.GAME/CRED:";
const char strCoinsPerCreditTwo[] PROGMEM = "2P.GAME/CRED:";
const char strPLevel[] PROGMEM ="P.SPEED:";



const char strWinner[] PROGMEM = "WINNER!";
const char strLoser[] PROGMEM = "LOSER!";
const char strCredits[] PROGMEM = "CREDIT(S)";
const char strWin[] PROGMEM = "WIN";
const char strP1Wins[] PROGMEM = "P1 WINS!";
const char strP2Wins[] PROGMEM = "P2 WINS!";

const char strStudent[] PROGMEM = "STUDENT";
const char strIntern[] PROGMEM = "INTERN";
const char strDoctor[] PROGMEM = "DOCTOR";
const char strSurgeon[] PROGMEM = "SURGEON";

const char strLevel[] PROGMEM = "LEVEL";


const char strGameMode[] PROGMEM = "2 PLAYER GAME";


const char strCopyright[] PROGMEM =    "2008 CODE CRANK";
const char strCopyrightTwo[] PROGMEM = "2016 GREG MAPES-ARCADE VER.1 ";
const char strWebsite[] PROGMEM =      "    HTTP://WWW.UZEBOX.ORG    "; //"HTTP://WWW.BELOGIC.COM/UZEBOX";
const char strLicence[] PROGMEM =      "  LICENCED UNDER GNU GPL V3  ";
const char strScoringOne[] PROGMEM =   " ONE PLAYER GAME:TIME BASED  ";
const char strScoringTwo[] PROGMEM =   "TWO PLAYER GAME:2 BUGS=2 JUNK";
const char strScoringThree[] PROGMEM = "3 BUGS=3 JUNKS 4 BUGS>=4 JUNK";
const char strGameOptions[] PROGMEM ="GAME OPTIONS";

const char strControlsOne[] PROGMEM =  "          -CONTROLS-         ";
const char strControlsTwo[] PROGMEM =  "W&B:ROTATE --- DOWN:MOVE DOWN";
const char strControlsThree[] PROGMEM ="                             ";


const char strReset[] PROGMEM = "HI SCORE RESET";
const char strMenu1PGame[] PROGMEM ="    1 PLAYER OR     ";
const char strMenu2PGame[] PROGMEM ="    2 PLAYER        ";
const char strMenuInsert[] PROGMEM ="INSERT";
const char strMenuCoin[] PROGMEM ="COIN(S)";
const char strMenuBlank[] PROGMEM ="                ";
const char strMenuMoreCoins[] PROGMEM ="COIN(S) FOR";


unsigned short HiScoreOne = 720;	
unsigned short HiScoreTwo = 480;
unsigned short HiScoreThree = 240;
unsigned short HiScoreFour = 960;

char NameOne[6] = "GREG.M";
char NameTwo[6] = "M.D.O.";
char NameThree[6] = "FRANK ";
char NameFour[6] = "D.PILL";
char PLevel = 1;

bool POneCoin = false;
bool PTwoCoin = false;
bool MovePill(unsigned short player, unsigned short move_type);
unsigned short grab_input(unsigned short player);
void NextPill(unsigned short player);
void NewPill(unsigned short player);
void LockPill(unsigned short player);
void InitBottle(unsigned short player);
void AnimateVirus();
void MainGameLoop();
void CheckInsertCoin(bool);
void playFx();

unsigned short CheckColorMatch(unsigned short tile1 , unsigned short tile2 );
void CheckMatching4(unsigned short player);
void SplitPill(unsigned short player,unsigned short x , unsigned short y);
void DropFloaters(unsigned short player);
void DropFilledSpace(unsigned short player,unsigned short x , unsigned short y);
void FlingTrash(unsigned short sender);
void QueTrash(unsigned short sender, unsigned short trash_count);



struct pill {
	bool orientation;
	unsigned short length;
	unsigned short x;
	unsigned short y;
	unsigned short bottle_x;
	unsigned short bottle_y;
	unsigned short top;
	unsigned short bottom;
	unsigned short left;
	unsigned short right;	
};

struct bottle {
	unsigned short space[BOTTLE_WIDTH][BOTTLE_DEPTH];
	unsigned short virus_count;
	bool check_for_sequences;
	unsigned short sequences_count;
	bool contains_floaters; 
	unsigned short dissapear_anim;
	unsigned short qued_trash;
	bool new_pill;
	unsigned short SeededNumber;
};


struct pill current_pill[2]; 
struct pill next_pill[2];
unsigned short gravity=60;
unsigned int round_time;
bool pill_falling=1;
unsigned short auto_repeat_counter=0;
struct bottle bottle[2];
unsigned short virus_anim_counter=0;
unsigned short drop_floater_timer=0;
unsigned short x,y,j,k,saved_state, z=0; // generic
unsigned char difficulty_level[2];
bool round_lost[2];
unsigned short rounds_won[2];
void SetStageSettings();
unsigned short move_type=0;
unsigned short seed;
void DissapearAnim(unsigned short player);
void RoundOver();
unsigned int long_timer;

bool WAIT_ON_NO_INPUT=0;
unsigned char GameNotOver = 0;
unsigned char credits = 0;
unsigned short StartingCounter = 1;
unsigned char coinsPerCreditOne = 1;
unsigned char coinsPerCreditTwo = 1;
bool vsMode=false; 			//two player mode
//bool restart=false;			//user choose to restart level
bool gameOver=true;			//gameover?
char sndFx = 0;

unsigned short POneGravityTime=0;
unsigned short PTwoGravityTime=0;
void SaveHighScore();
void LoadHighScore();
void OptionsMenu();
void SinglePlayerMenu();
void PrintHiScores();
void NewHiScore();


void DrawMainMenu(){
	SetTileTable(main_board);
	DrawMap(0,0,drmario_map_stage_select);
}

int main(){

   	ClearVram();
	SetFontTable(fonts);
	InitMusicPlayer(patches);
	WaitVsync(30);
	if(ReadJoypad(0)&BTN_SL)
	{
		SaveHighScore();
		Print(14,14,strReset);
		while(ReadJoypad(0)!=0); //wait for key release
	}
	LoadHighScore(); // Load High Score after checking for reset...

DrawMainMenu();


//	SetTileTable(MDTiles);
//	DrawMap(26,9,md_title);

while (1)
{
	WaitVsync(1);
	playFx();

	StopSong(NextSong);
	Print(17,18,strCredits);
//	rnd+=13;
		if ((credits < coinsPerCreditOne) && ((StartingCounter%1000) < 500))
		{
			PrintChar(19,13,48 + coinsPerCreditOne - credits); 
			Print(12,13,strMenuInsert);
			Print(21,13,strMenuCoin);
		}

		else if ((credits < coinsPerCreditOne) && ((StartingCounter % 1000) >= 500 )) Print(12,14,strMenuBlank);

		else if (credits < coinsPerCreditOne + coinsPerCreditTwo)
		{
	        PrintChar(18,14,48+ coinsPerCreditOne + coinsPerCreditTwo - credits); 
			Print(11,13,strMenu1PGame);
			Print(11,14,strMenuInsert);
			Print(20,14,strMenuMoreCoins);
			Print(11,15,strMenu2PGame);
		}
		else 
		{
			Print(11,13,strMenu1PGame);
			Print(11,14,strMenu2PGame);
			Print(15,15,strMenuBlank);
		}
		if((ReadJoypad(0)&BTN_START) && (credits >= coinsPerCreditOne))
		{
			while(ReadJoypad(0)!=0); //wait for key release
			credits-=coinsPerCreditOne;
			GameNotOver = true;
			vsMode=false;
			MainGameLoop();
		}
		if(ReadJoypad(0)&BTN_SR) SoftReset();
		if(ReadJoypad(0)&BTN_SL)
		{
			OptionsMenu();
		}

		if((ReadJoypad(0)&BTN_SELECT) && (credits >= coinsPerCreditOne + coinsPerCreditTwo))
		{
			while(ReadJoypad(0)!=0); //wait for key release
			credits -= (coinsPerCreditOne + coinsPerCreditTwo);
			GameNotOver = true;
			vsMode=true;
			MainGameLoop();
		}

		// Show directions screen
		if (StartingCounter == 20000){
			Print(6,20,strControlsOne);
			Print(6,21,strControlsTwo);
			Print(6,22,strControlsThree);
		}
		else if (StartingCounter == 10000){
			Print(6,20,strScoringOne);
			Print(6,21,strScoringTwo);
			Print(6,22,strScoringThree);
		}
		else if (StartingCounter == 1){
			Print(6,20,strCopyrightTwo);
			Print(6,21,strLicence);
			Print(6,22,strWebsite);
		}


		// Insert coin routine...
		CheckInsertCoin(true);
//		srand(rnd);			
		if (StartingCounter > 30000) StartingCounter = 1;
		else StartingCounter++;	
	}
}

unsigned short Randomize(unsigned short player){
	
	unsigned short lsb;
    lsb = bottle[player].SeededNumber & 1;		   /* Get LSB (i.e., the output bit). */
    bottle[player].SeededNumber >>= 1;              /* Shift register */
   	bottle[player].SeededNumber ^= (-lsb) & 0xB400u;/* If the output bit is 1, apply toggle mask.
//       			                        * The value has 1 at bits corresponding
//                   			            * to taps, 0 elsewhere. */

	return bottle[player].SeededNumber;
}



void CheckInsertCoin(bool MenuScreen){

		if(ReadJoypad(1)&BTN_SR && (POneCoin == false))
		{
			credits++;
			sndFx |= BIT_1;
			POneCoin = true;
		}
		else if(!(ReadJoypad(1)&BTN_SR) && POneCoin == true)
			POneCoin = false;
		if(ReadJoypad(1)&BTN_SL && (PTwoCoin == false))
		{
			credits++;
			sndFx |= BIT_1;
			PTwoCoin = true;
		}
		else if(!(ReadJoypad(1)&BTN_SL) && PTwoCoin == true)
			PTwoCoin = false;
		if (MenuScreen) PrintInt(15,18,credits, false);

}


void MainGameLoop(){
 
	difficulty_level[PLAYER1]=1;
	difficulty_level[PLAYER2]=1;


	unsigned short i=0;
//	unsigned short j=0;
//	SetTileTable(main_board);	 
	DrawMap(6,13,drmario_blackout_menu);
	

	while (GameNotOver) 
	{
		
		if (!vsMode)
		{
			SinglePlayerMenu();
		}
		else{
		SetStageSettings();
		}
//		SetTileTable(pills);

		rounds_won[PLAYER1]=0;
		rounds_won[PLAYER2]=0;
		//srand(seed);

		while (( rounds_won[PLAYER1] < 3) && ( rounds_won[PLAYER2] < 3) && (GameNotOver))
		{	

			round_lost[PLAYER1]=0;
			round_lost[PLAYER2]=0;

		 	ClearVram();
			SetTileTable(main_board);	 
			if (vsMode) DrawMap(0,0,drmario_map_main);
			else {
				DrawMap(0,0,drmario_map_main_single);
				Print(18,22,strBugs); 
				Print(17,9,strLevel); 
//				Print(17,2,strTimeToBeat);
				PrintHiScores(); 
				Print(19, 5,strM);
				Print(18,2,strLevel);
				if (difficulty_level[PLAYER1]==6) Print(17,4,strStudent);
				if (difficulty_level[PLAYER1]==11) Print(17,3,strIntern);
				if (difficulty_level[PLAYER1]==16)Print(17,3,strDoctor);
				if (difficulty_level[PLAYER1]==21)Print(17,3,strSurgeon);
				}


			// redraw score board
			for (i=0 ; i < rounds_won[PLAYER1] ; i++ )
//				Print(16,18-(i*3),strWin); 
				DrawMap(16,17-(i*3),checkMark);
			for (i=0 ; i < rounds_won[PLAYER2] ; i++ )
				DrawMap(21,17-(i*3),checkMark);
//				Print(21,18-(i*3),strWin); 



			SetTileTable(pills);
	
			StartSong(song_drmario_main);
		
			InitBottle(PLAYER1);
			NextPill(PLAYER1);
			NewPill(PLAYER1);		
			POneGravityTime = 0;
			PTwoGravityTime = 0;
			if (vsMode)
			{
				InitBottle(PLAYER2);
				NextPill(PLAYER2);
				NewPill(PLAYER2);
			}

			round_time = 0;
			i=0;
			while ( ! ( round_lost[PLAYER1] || round_lost[PLAYER2] ) )
			{		

		//		if(GetVsyncFlag()) // thx uze !
		//		{

		//			ClearVsyncFlag();
					WaitVsync(1);
					playFx();
					// you win if all viruses are gone
					if ( bottle[PLAYER1].virus_count == 0 )
					{
						// player 2 loses
						round_lost[PLAYER2]=1;				
					}
					if ( bottle[PLAYER2].virus_count == 0 && vsMode)
										{
						// player 1 loses
						round_lost[PLAYER1]=1;
					}
					if (!vsMode){
						PrintByte(18,5,round_time/3600, false);
						PrintByte(22,5,(round_time/60)%60, false);
					}
					// check for squences if needed
					while  ( bottle[PLAYER1].check_for_sequences )
					{
						CheckMatching4(PLAYER1);
					}	

					while  ( bottle[PLAYER2].check_for_sequences && vsMode)
					{
						CheckMatching4(PLAYER2);
					}	

					// always anymate virus
					AnimateVirus();
			

					// drop floaters unless pills dissapear ainmation is going on
					if ( drop_floater_timer < 10 ) 
					{
						drop_floater_timer++;
					}
					else
					{
						drop_floater_timer=0;
	
						if ( bottle[PLAYER1].contains_floaters ) 
						{
							 if ( ! bottle[PLAYER1].dissapear_anim )
							 	DropFloaters(PLAYER1);
						}	

						if ( bottle[PLAYER2].contains_floaters && vsMode)
						{
							if ( ! bottle[PLAYER2].dissapear_anim )
								DropFloaters(PLAYER2);	
						}
					}
				
 	
	
					// until gravity timer hits
					 // move pill around unless some pills are dissapearing or there are floaters
					if ( bottle[PLAYER1].dissapear_anim) 
					{		
						DissapearAnim(PLAYER1);
					}	
					else if (bottle[PLAYER1].contains_floaters)
					{
						POneGravityTime=0;
					}
					else if (bottle[PLAYER1].new_pill == true)
					{ 
						if (bottle[PLAYER1].qued_trash > 0) 
						{
							FlingTrash(PLAYER1);
						}
						else
						{								
							NewPill(PLAYER1);
							bottle[PLAYER1].new_pill = false;
							POneGravityTime=0;
						}							
					}
					else if ((move_type=grab_input(PLAYER1)))
					{
						MovePill(PLAYER1, move_type);
					}
					else if ( POneGravityTime > gravity - 6 * (round_time/(1800 * (8-PLevel)/3)))
					{
						// force move down when time hits
						MovePill(PLAYER1, MOVE_DOWN);
						POneGravityTime=0;  //Not needed anymore?
					}

					if (vsMode){

					if ( bottle[PLAYER2].dissapear_anim) 
					{
							DissapearAnim(PLAYER2);
					}
					else if ( bottle[PLAYER2].contains_floaters )
					{
						PTwoGravityTime=0;
					}
					else if (bottle[PLAYER2].new_pill == true)
					{ 
						if (bottle[PLAYER2].qued_trash > 0) 
						{
							FlingTrash(PLAYER2);
						}
						else
						{								
							NewPill(PLAYER2);
							bottle[PLAYER2].new_pill = false;
							PTwoGravityTime = 0;
						}
					}
					else if ((move_type=grab_input(PLAYER2)))
					{
						MovePill(PLAYER2, move_type);
					}
					else if ( PTwoGravityTime > gravity - 6 * (round_time/(1800 * (8-PLevel)/3)))
					{
						// force move down when time hits
						MovePill(PLAYER2, MOVE_DOWN);
						PTwoGravityTime=0;
					}


					PTwoGravityTime++;
					}
				POneGravityTime++;
				round_time++;

				//}

			}

			RoundOver();
 	
		}


	} // end while (1)
	
	DrawMainMenu();


}  // Loop back to main screen.
	
//} 


void RoundOver()
{
			POneGravityTime = 0;
			SetTileTable(main_board);	 
			StopSong(song_drmario_main);

			if ( round_lost[PLAYER1] )
			{
			
 				rounds_won[PLAYER2]++;

				// Check for VS mode, update score board
				if (vsMode)
				{
	 				Print(BOTTLE_FIRST_COLUMN+1, BOTTLE_FIRST_COLUMN+5, strLoser);
					Print(BOTTLE_FIRST_COLUMN+25, BOTTLE_FIRST_COLUMN+5, strWinner);
//					Print(21,21-(rounds_won[PLAYER2]*3),strWin);
					StartSong(NextSong);
					DrawMap(21,20-(rounds_won[PLAYER2]*3),checkMark);

					if ( rounds_won[PLAYER2] == 3 )
					{ 
						sndFx |= BIT_3;
						Print(16,9,strP2Wins);	
						GameNotOver = 0;	
					}
				}
				else
				{
					sndFx |= BIT_5;
					GameNotOver = 0;	
				}	
 		   	}
			else
			{
				Print(BOTTLE_FIRST_COLUMN+1, BOTTLE_FIRST_COLUMN+5, strWinner);
				if (vsMode) Print(BOTTLE_FIRST_COLUMN+25, BOTTLE_FIRST_COLUMN+5, strLoser);
	 			rounds_won[PLAYER1]++;
				sndFx |= BIT_5;
 				StartSong(NextSong);
				// score board
				//Print(16,21-(rounds_won[PLAYER1]*3),strWin);
				DrawMap(16,20-(rounds_won[PLAYER1]*3),checkMark);

				if ( rounds_won[PLAYER1] == 3 )
				{ 
					Print(16,9,strP1Wins);
					GameNotOver = 0;	
				}
			}

			
			long_timer=10000;
			// make "press start" blink untill someone presses start
			while (POneGravityTime < 10)
			{
			StartingCounter++;
				if (   long_timer == 20000 ) 
				{
					Print(BOTTLE_FIRST_COLUMN+2, BOTTLE_FIRST_COLUMN+11, strPress);
					Print(BOTTLE_FIRST_COLUMN+2, BOTTLE_FIRST_COLUMN+12, strStart);
					if (vsMode)
					{
     					Print(BOTTLE_FIRST_COLUMN+26, BOTTLE_FIRST_COLUMN+11, strPress);
						Print(BOTTLE_FIRST_COLUMN+26, BOTTLE_FIRST_COLUMN+12, strStart);
					}
					else if (difficulty_level[PLAYER1] == 16 && round_time/60 < HiScoreOne && bottle[PLAYER1].virus_count == 0)
					    {
						NewHiScore();
						POneGravityTime = 10;
						}
					else if (round_time/60 < HiScoreTwo && difficulty_level[PLAYER1] == 11 && bottle[PLAYER1].virus_count == 0){
						NewHiScore();
						POneGravityTime = 10;
						}
					else if (round_time/60 < HiScoreThree && difficulty_level[PLAYER1] == 6 && bottle[PLAYER1].virus_count == 0){
						NewHiScore();
						POneGravityTime = 10;
						}
					else if (round_time/60 < HiScoreFour && difficulty_level[PLAYER1] == 21 && bottle[PLAYER1].virus_count == 0){
						NewHiScore();
						POneGravityTime = 10;
						}
					else if (GameNotOver == 0)
					{

						Print(BOTTLE_FIRST_COLUMN+9, BOTTLE_FIRST_COLUMN+9,	strMenuBlank);
						Print(BOTTLE_FIRST_COLUMN+9, BOTTLE_FIRST_COLUMN+10,strMenuBlank);
						Print(BOTTLE_FIRST_COLUMN+9, BOTTLE_FIRST_COLUMN+11,strMenuBlank);
						Print(BOTTLE_FIRST_COLUMN+10, BOTTLE_FIRST_COLUMN+10, strGame);
						Print(BOTTLE_FIRST_COLUMN+20, BOTTLE_FIRST_COLUMN+10, strOver);
					}

					
				}

				if (  long_timer == 40000 )
				{

					Print(BOTTLE_FIRST_COLUMN+2, BOTTLE_FIRST_COLUMN+11, strBlank);
					Print(BOTTLE_FIRST_COLUMN+2, BOTTLE_FIRST_COLUMN+12, strBlank);
					if (vsMode)
					{
     					Print(BOTTLE_FIRST_COLUMN+26, BOTTLE_FIRST_COLUMN+11, strBlank);
						Print(BOTTLE_FIRST_COLUMN+26, BOTTLE_FIRST_COLUMN+12, strBlank);
					}
				long_timer=0;
				if (GameNotOver == 0) POneGravityTime++;
				}				
				if (ReadJoypad(PLAYER1) & BTN_START ) POneGravityTime = 10;
				long_timer++;
			};

			while ( ReadJoypad(PLAYER1) !=0);

			// Increment Level
			if ((!vsMode) && (difficulty_level[PLAYER1] <20)) difficulty_level[PLAYER1] +=5;
			
}


void DissapearAnim(unsigned short player)
{
	bottle[player].dissapear_anim=0;

	unsigned short first_column;


	if ( player == PLAYER1 ) 
	{
		first_column=BOTTLE_FIRST_COLUMN;
	}
	else
	{
		first_column=BOTTLE2_FIRST_COLUMN;
	}


	for ( x=0 ; x< BOTTLE_WIDTH ; x++ )
	{
		for ( y=0 ; y< BOTTLE_DEPTH ; y++ )
		{
			if ( (  bottle[player].space[x][y] >= DISSAPEAR_ANIM_1_R ) && ( bottle[player].space[x][y] <= DISSAPEAR_ANIM_3_R ) )
			{
				bottle[player].space[x][y]++;
				SetTile(first_column+x, BOTTLE_FIRST_ROW+y, bottle[player].space[x][y]);
				bottle[player].dissapear_anim=1;
			}
			else if ( bottle[player].space[x][y]  == DISSAPEAR_ANIM_4_R )
			{
				bottle[player].space[x][y]=0;
				SetTile(first_column+x, BOTTLE_FIRST_ROW+y, bottle[player].space[x][y]);
				bottle[player].dissapear_anim=1;
			}
			else if ( (  bottle[player].space[x][y] >= DISSAPEAR_ANIM_1_B ) && ( bottle[player].space[x][y] <= DISSAPEAR_ANIM_3_B ) )
			{
				bottle[player].space[x][y]++;
				SetTile(first_column+x, BOTTLE_FIRST_ROW+y, bottle[player].space[x][y]);
				bottle[player].dissapear_anim=1;
			}
			else if ( bottle[player].space[x][y]  == DISSAPEAR_ANIM_4_B )
			{
				bottle[player].space[x][y]=0;
				SetTile(first_column+x, BOTTLE_FIRST_ROW+y, bottle[player].space[x][y]);
				bottle[player].dissapear_anim=1;
			}
			else if ( (  bottle[player].space[x][y] >= DISSAPEAR_ANIM_1_Y ) && ( bottle[player].space[x][y] <= DISSAPEAR_ANIM_3_Y ) )
			{
				bottle[player].space[x][y]++;
				SetTile(first_column+x, BOTTLE_FIRST_ROW+y, bottle[player].space[x][y]);
				bottle[player].dissapear_anim=1;
			}
			else if ( bottle[player].space[x][y]  == DISSAPEAR_ANIM_4_Y )
			{
				bottle[player].space[x][y]=0;
				SetTile(first_column+x, BOTTLE_FIRST_ROW+y, bottle[player].space[x][y]);
				bottle[player].dissapear_anim=1;
			}
		}
	}
}

void SetStageSettings()
{

	unsigned int c;

	DrawMap(8,16,game_vs_menu);


	Print(14, 13, strGameMode);
	Print(7, 15, strLevel);

	SetTile(8+difficulty_level[PLAYER1],18,54);
	SetTile(8+difficulty_level[PLAYER2],20,55);

	PrintByte(32,18,difficulty_level[PLAYER1],false);
	PrintByte(32,20, difficulty_level[PLAYER2],false);


	while (1)
	{
		
		seed++;
		
		c = ReadJoypad(PLAYER1);

		if (c) 
		{
			while (	ReadJoypad(PLAYER1)!=0 );
		}


		if ( ( c&BTN_RIGHT ) && ( difficulty_level[PLAYER1] < 20 ) )
		{
			SetTile(8+difficulty_level[PLAYER1],18,0);
			difficulty_level[PLAYER1]++;
			SetTile(8+difficulty_level[PLAYER1],18,54);
			PrintByte(32,18,difficulty_level[PLAYER1],false);
		}
		else if ( ( c&BTN_LEFT) && ( difficulty_level[PLAYER1] > 1 ) )
		{
			SetTile(8+difficulty_level[PLAYER1],18,0);
			difficulty_level[PLAYER1]--;
			SetTile(8+difficulty_level[PLAYER1],18,54);
			PrintByte(32,18, difficulty_level[PLAYER1],false);
		}
		else if ( c&BTN_START )
			break; 

		c = ReadJoypad(PLAYER2);
		
		if ( c ) 
		{
			while (	ReadJoypad(PLAYER2)!=0 );
		}


		if ( (  c&BTN_RIGHT ) && ( difficulty_level[PLAYER2] < 20 ) )
		{
			SetTile(8+difficulty_level[PLAYER2],20,0);
			difficulty_level[PLAYER2]++;
			SetTile(8+difficulty_level[PLAYER2],20,55);
			PrintByte(32,20,difficulty_level[PLAYER2],false);
		}
		else if ( ( c&BTN_LEFT ) && ( difficulty_level[PLAYER2] > 1 ) )
		{
			SetTile(8+difficulty_level[PLAYER2],20,0);
			difficulty_level[PLAYER2]--;
			SetTile(8+difficulty_level[PLAYER2],20,55);
			PrintByte(32,20, difficulty_level[PLAYER2],false);
		}
		else if ( c&BTN_START )
			break; 

			
	}


}


bool MovePill(unsigned short player, unsigned short move_type)
{

	bool moved=0;

	switch (move_type ) {


		case MOVE_DOWN:
	


			if ( current_pill[player].y ==  BOTTLE_LAST_ROW ) 
			{
				LockPill(player);
				bottle[player].new_pill = true;

				break;
			}
			
			if ( current_pill[player].orientation == HORIZ )
			{
				if (  bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y+1] 
					||  bottle[player].space[current_pill[player].bottle_x+1][current_pill[player].bottle_y+1]  ) 
				{
					
					LockPill(player);
					bottle[player].new_pill = true;


				}
				else
				{		

					Fill(current_pill[player].x,current_pill[player].y, current_pill[player].length, 1, 0);
					current_pill[player].y++;
					current_pill[player].bottle_y++;
			    	SetTile(current_pill[player].x,current_pill[player].y,  current_pill[player].left);
			    	SetTile(current_pill[player].x+1,current_pill[player].y,  current_pill[player].right);
					moved=1;	
				}		
			}
			else
			{
				//  something below me ?
				if ( bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y+1] )
				{
					LockPill(player);
					bottle[player].new_pill = true;
				}
				else
				{		
				 	Fill(current_pill[player].x, ( current_pill[player].y - current_pill[player].length + 1 ), 1, 1, 0);
					current_pill[player].y++;
					current_pill[player].bottle_y++;
					SetTile(current_pill[player].x,current_pill[player].y-1, current_pill[player].top);
					SetTile(current_pill[player].x,current_pill[player].y,  current_pill[player].bottom);	
					moved=1;
				}
			}
		if(player==0) POneGravityTime = 0;
		else PTwoGravityTime = 0;


		
			break;

		case MOVE_LEFT:	

			// first column , can't move left
			if ( (current_pill[player].bottle_x == 0) )
				break ;


			if ( current_pill[player].orientation == HORIZ )
			{
				//  nothing left of me ?
				if ( ! bottle [player].space[current_pill[player].bottle_x-1][current_pill[player].bottle_y] ) 
				{

					Fill(current_pill[player].x,current_pill[player].y, current_pill[player].length, 1, 0);			
					current_pill[player].x--;
					current_pill[player].bottle_x--;
					SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].left);
					SetTile(current_pill[player].x+1,current_pill[player].y, current_pill[player].right);
					moved=1;
				}
			}
			else
			{
 				//  nothing left of me ?
				if ( ! ( bottle[player].space[current_pill[player].bottle_x-1][current_pill[player].bottle_y] 
					 || bottle[player].space[current_pill[player].bottle_x-1][current_pill[player].bottle_y-1] ) ) 
				{


					Fill(current_pill[player].x, current_pill[player].y-1, 1,current_pill[player].length , 0);
					current_pill[player].x--;
					current_pill[player].bottle_x--;
					SetTile(current_pill[player].x,current_pill[player].y-1, current_pill[player].top);
					SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].bottom);
					moved=1;	
				}
			}

			break;


		case MOVE_RIGHT:	


			if ( current_pill[player].orientation == HORIZ )
			{
				if  ( current_pill[player].bottle_x  == (BOTTLE_WIDTH - 2) )
					break;

				//  nothing right of me ?
				if ( ! bottle[player].space[current_pill[player].bottle_x+2][current_pill[player].bottle_y] ) 
				{

					SetTile(current_pill[player].x, current_pill[player].y, 0);
					current_pill[player].x++;
					current_pill[player].bottle_x++;
					SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].left);
					SetTile(current_pill[player].x+1,current_pill[player].y, current_pill[player].right);
					moved=1;
				}

			}
			else	// VERTICAL 
			{
				if (current_pill[player].bottle_x == (BOTTLE_WIDTH - 1) ) 
					break;
									
				//  nothing right of me ?
				if ( ! ( bottle[player].space[current_pill[player].bottle_x+1][current_pill[player].bottle_y] 
						|| bottle[player].space[current_pill[player].bottle_x+1][current_pill[player].bottle_y-1])  ) 
				{


					Fill(current_pill[player].x, current_pill[player].y-1 , 1, current_pill[player].length, 0);
					current_pill[player].x++;
					current_pill[player].bottle_x++;
					SetTile(current_pill[player].x,current_pill[player].y-1, current_pill[player].top);
					SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].bottom);	
					moved=1;
				}
			}

			break;

		case MOVE_SPIN_C:	
		case MOVE_SPIN_C_C:	

			if ( current_pill[player].orientation == HORIZ )
			{
				// check for possible ceiling kicks
				if (  current_pill[player].bottle_y == 0 ) 
				{
						if ( ! MovePill(player, MOVE_DOWN) )
							break ;	
				}					
				if  ( bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y-1]  ) 
				{	  
						if ( ! MovePill(player, MOVE_DOWN) )
							break;
				}	
					
				Fill(current_pill[player].x,current_pill[player].y, 2, 1, 0);
				if (move_type == MOVE_SPIN_C){
				current_pill[player].bottom = current_pill[player].left;
				current_pill[player].top = current_pill[player].right;
			
				current_pill[player].top +=2 ; // left to top pill tile
				current_pill[player].bottom += 2; // right to bottom pile tile 

//				SetTile(current_pill[player].x,current_pill[player].y-1, current_pill[player].top);
//				SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].bottom);
				}
				else{
				current_pill[player].bottom = current_pill[player].right;
				current_pill[player].top = current_pill[player].left;
			
				current_pill[player].top +=3 ; // left to top pill tile
				current_pill[player].bottom +=1; // right to bottom pile tile 
				}

				SetTile(current_pill[player].x,current_pill[player].y-1, current_pill[player].top);
				SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].bottom);
				


				moved=1;

				current_pill[player].orientation = VERT;
			}
			else // VERTICAL 
			{

				
				// check for possible wall kicks to the left
							
				// against the wall or pils ?
				if  (  ( current_pill[player].bottle_x == BOTTLE_WIDTH-1  ) 
					 ||( bottle[player].space[current_pill[player].bottle_x+1][current_pill[player].bottle_y] != 0 ) )
				{	  
						if ( ! MovePill(player, MOVE_LEFT) )
						{
							// special squeeze kick possible ?
							if (  bottle[player].space[current_pill[player].bottle_x-1][current_pill[player].bottle_y] == 0 )
							{
								// can't go through bottle wall...
								if ( current_pill[player].bottle_x == 0 )
									break;
								if (move_type == MOVE_SPIN_C){
								current_pill[player].left =  current_pill[player].top;
				 				current_pill[player].right = current_pill[player].bottom;

								current_pill[player].left -= 3; // rotated tile
								current_pill[player].right -= 1; // rotated tile 
								}
								else{
								current_pill[player].left =  current_pill[player].bottom;
				 				current_pill[player].right = current_pill[player].top;

								current_pill[player].left -= 2; // rotated tile
								current_pill[player].right -= 2; // rotated tile 
								}

								SetTile (current_pill[player].x,current_pill[player].y-1,0);
								
								current_pill[player].x--;
								current_pill[player].bottle_x--;
								SetTile(current_pill[player].x,current_pill[player].y,  current_pill[player].left);
    							SetTile(current_pill[player].x+1,current_pill[player].y, current_pill[player].right);

								current_pill[player].orientation = HORIZ;
								moved=1;
								
							}

							break;

						}
					
				} 
		
			


				Fill(current_pill[player].x, ( current_pill[player].y - 1 ), 1, 2, 0);

			if (move_type == MOVE_SPIN_C){

				current_pill[player].left =  current_pill[player].top;
				current_pill[player].right = current_pill[player].bottom;

				current_pill[player].left -= 3; // rotated tile
				current_pill[player].right -= 1; // rotated tile 
				}
				else{
				current_pill[player].left =  current_pill[player].bottom;
				current_pill[player].right = current_pill[player].top;

				current_pill[player].left -= 2; // rotated tile
				current_pill[player].right -= 2; // rotated tile 
				}

				SetTile(current_pill[player].x,current_pill[player].y,  current_pill[player].left);
    			SetTile(current_pill[player].x+1,current_pill[player].y, current_pill[player].right);

				current_pill[player].orientation = HORIZ;
				moved=1;
			}

			break;		

	}


	return moved;


}


void NextPill(unsigned short player)
{
	z= Randomize(player) % 9;

	// random 1, 12 ,23
	next_pill[player].left= z/3 * 11 + 1 ;	

	// random 2, 13, 24
	next_pill[player].right= z%3 * 11 + 2 ;	

	if ( player == PLAYER1 )
	{
		next_pill[player].x = 7;
	}
	else
	{
		next_pill[player].x = 7+24;
	}
	next_pill[player].y = BOTTLE_FIRST_ROW -2 ;

			
	SetTile(next_pill[player].x,next_pill[player].y,  next_pill[player].left);
    SetTile(next_pill[player].x+1,next_pill[player].y, next_pill[player].right);

}

void NewPill(unsigned short player)
{

	
// check if bottle if full

	if ( ( bottle[player].space[3][0] ) || ( bottle[player].space[4][0] ) )
	{	
		round_lost[player]=1;
		return;
	}

	if ( player == PLAYER1 )
	{
		current_pill[player].x = 7;
	}
	else
	{
		current_pill[player].x = 7+24;
	}

	current_pill[player].bottle_x=3;	
	current_pill[player].y = BOTTLE_FIRST_ROW ;	
	current_pill[player].bottle_y=0;


	current_pill[player].orientation = 0;
	current_pill[player].length = 2;

	current_pill[player].left = next_pill[player].left;
	current_pill[player].right = next_pill[player].right;
    
	Fill(current_pill[player].x,current_pill[player].y, 1, 1, current_pill[player].left);
    Fill(current_pill[player].x+1,current_pill[player].y, 1, 1, current_pill[player].right);

 	NextPill(player);
				
}

void LockPill(unsigned short player)
{
	if ( current_pill[player].orientation == HORIZ )
	{
		bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y]=current_pill[player].left;
		bottle[player].space[current_pill[player].bottle_x+1][current_pill[player].bottle_y]=current_pill[player].right;
	}
	else
	{
		// use horizontal tiles for saving color
		bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y-1]=current_pill[player].top;
		bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y]=current_pill[player].bottom;
	}
	
	bottle[player].check_for_sequences=1;
	
}



void CheckMatching4(unsigned short player)
{	

	short unsigned int matching_spaces = 1;
	unsigned short first_column; 
	unsigned short virus_left_column;


	bottle[player].check_for_sequences=0;

	if ( player == PLAYER1 ) 
	{
		first_column=BOTTLE_FIRST_COLUMN;
		virus_left_column=VIRUS_LEFT_COLUMN1; 
	}
	else
	{
		first_column=BOTTLE2_FIRST_COLUMN;
		virus_left_column=VIRUS_LEFT_COLUMN2;
	}

	if (bottle[player].dissapear_anim==0)
	{
		for ( x=0 ; x< BOTTLE_WIDTH ; x++ ) // attempting to +1 bottle width check
		{
			for ( y=BOTTLE_DEPTH ; y> 0 ; y-- ) // y=0 ; y< BOTTLE_DEPTH ; y++
			{
				// for evey non blank space
				if ( bottle[player].space[x][y] )
				{
				// check remaining of column for matching colors				
					matching_spaces=1;
					for ( j=0; j < BOTTLE_DEPTH ; j++ ) // was < 10/30/20
					{
	//					if ( y-j >= BOTTLE_DEPTH ) // was >= BOTTLE_DEPTH 10/30/20
	//						break; // don't check passed last row
//						else if ( y-j-1 > BOTTLE_DEPTH ) // was >= BOTTLE_DEPTH 10/30/20
//							break; // don't check passed last row

    					if ( (CheckColorMatch ( bottle[player].space[x][y], bottle[player].space[x][y-j-1])) && ( y-j-1 <= BOTTLE_DEPTH ) ) // was y+j+1 on 1/29/20
						{
							matching_spaces++;
						}
						else 
						{
							// are we at the end of a matching sequence  ?
							if ( matching_spaces >= 4 )
							{
								// don't need j anymore
								for ( j=0 ; j< matching_spaces ; j++)
								{

									SplitPill(player,x,y-j);
									bottle[player].contains_floaters =1;	
									// update virus count is needed
									switch (bottle[player].space[x][y-j])
										{
										// right half
											case VIRUS_RED1 ... VIRUS_RED2:
											case VIRUS_BLUE1 ... VIRUS_BLUE2:
											case VIRUS_YELLOW1 ... VIRUS_YELLOW2:
												bottle[player].virus_count--;
												PrintByte(virus_left_column, VIRUS_LEFT_ROW, bottle[player].virus_count,false);
												break;
										}
//									if ( ( bottle[player].space[x][y+j] >= VIRUS_RED1 )  ( bottle[player].space[x][y+j] <= VIRUS_YELLOW2 ) )
//									{
//										bottle[player].virus_count--;
//										PrintByte(virus_left_column, VIRUS_LEFT_ROW, bottle[player].virus_count,false);
//
//									}
									if ((bottle[player].space[x][y-j] > 0) && (bottle[player].space[x][y-j] < 9)) 
									{ 
										bottle[player].space[x][y-j]=DISSAPEAR_ANIM_1_R;
										Fill(first_column+x,BOTTLE_FIRST_ROW+y-matching_spaces+1,1,matching_spaces,DISSAPEAR_ANIM_1_R);
									}
									else if ((bottle[player].space[x][y-j] > 11) && (bottle[player].space[x][y-j] < 20)) 
									{ 
										bottle[player].space[x][y-j]=DISSAPEAR_ANIM_1_B;
										Fill(first_column+x,BOTTLE_FIRST_ROW+y-matching_spaces+1,1,matching_spaces,DISSAPEAR_ANIM_1_B);
									}
									else
									{
										bottle[player].space[x][y-j]=DISSAPEAR_ANIM_1_Y;
										Fill(first_column+x,BOTTLE_FIRST_ROW+y-matching_spaces+1,1,matching_spaces,DISSAPEAR_ANIM_1_Y);
									}
								}

							    sndFx |= BIT_2;
/*
1,2,7,8,13,16,17 = red
3,4,9,10,14,18,19 = blue
5,6,11,12,15,20,21 = yellow
*/
								// start pill dissapear animation sequence

								bottle[player].dissapear_anim=1;

								bottle[player].check_for_sequences=1;
								bottle[player].sequences_count++;

							}
							// stop searching column
							break;
						}
					}

				// check remaining of row for matching colors
						
				matching_spaces=1;
				for ( j=0; j < BOTTLE_WIDTH ; j++ )
				{
						if ( x+j >= BOTTLE_WIDTH )
							break; // don't check passed last row

    					else if ( CheckColorMatch ( bottle[player].space[x][y], bottle[player].space[x+j+1][y]) ) // was x+j+1 on 1/29/20
						{
							matching_spaces++;
						}
						else 
						{
							// are we at the end of a matching sequence  ?
							if ( matching_spaces >= 4 )
							{
							
								// don't need j anymore, let;s uze it for something
								for ( j=0 ; j< matching_spaces ; j++)
								{
									// remove pill(s)/parts from bottle 
									SplitPill(player, x+j,y);
									bottle[player].contains_floaters =1;
									// update virus count is needed
									switch (bottle[player].space[x+j][y])
										{
										// right half
											case VIRUS_RED1 ... VIRUS_RED2:
											case VIRUS_BLUE1 ... VIRUS_BLUE2:
											case VIRUS_YELLOW1 ... VIRUS_YELLOW2:
												bottle[player].virus_count--;
												PrintByte(virus_left_column, VIRUS_LEFT_ROW, bottle[player].virus_count,false);
												break;
										}

//									if ( ( bottle[player].space[x+j][y] >= VIRUS_RED1 ) && ( bottle[player].space[x+j][y] <= VIRUS_YELLOW2 ) )
//									{
//										bottle[player].virus_count--;
//										PrintByte(virus_left_column, VIRUS_LEFT_ROW, bottle[player].virus_count,false);
//									}
									
						

//									bottle[player].space[x+j][y]=DISSAPEAR_ANIM_1;

								if ((bottle[player].space[x+j][y] > 0) && (bottle[player].space[x+j][y] < 9)) {
								    bottle[player].space[x+j][y]=DISSAPEAR_ANIM_1_R;
									Fill(first_column+x,BOTTLE_FIRST_ROW+y,matching_spaces,1,DISSAPEAR_ANIM_1_R);
								}
								else if ((bottle[player].space[x+j][y] > 11) && (bottle[player].space[x+j][y] < 20)) { 
									bottle[player].space[x+j][y]=DISSAPEAR_ANIM_1_B;
									Fill(first_column+x,BOTTLE_FIRST_ROW+y,matching_spaces,1,DISSAPEAR_ANIM_1_B);
								}
								else {
									bottle[player].space[x+j][y]=DISSAPEAR_ANIM_1_Y;
									Fill(first_column+x,BOTTLE_FIRST_ROW+y,matching_spaces,1,DISSAPEAR_ANIM_1_Y);
								}

									
								}

								sndFx |= BIT_2;
								// black out matching color sequence
							
/*
Fill(first_column+x,BOTTLE_FIRST_ROW+y,matching_spaces,1,DISSAPEAR_ANIM_1);

1,2,7,8,13,16,17 = red
3,4,9,10,14,18,19 = blue
5,6,11,12,15,20,21 = yellow
*/
								// start pill dissapear animation sequence
								bottle[player].dissapear_anim=1;

								bottle[player].check_for_sequences=1;
								bottle[player].sequences_count++;
							}
	
							// stop searching column

							break;
						}
				}
			}
		}
	}
	}
}




void SplitPill(unsigned short player, unsigned short x , unsigned short y)
{

	unsigned short first_column; 


	if ( player == PLAYER1 ) 
	{
		first_column=BOTTLE_FIRST_COLUMN;
	}
	else
	{
		first_column=BOTTLE2_FIRST_COLUMN;
	}



	bottle[player].check_for_sequences=0;

	switch ( bottle[player].space[x][y] )
	{
		// half pills
		case SINGLE_RED:
		case SINGLE_BLUE:
		case SINGLE_YELLOW :		
			break;

		// left half
		case LEFT_RED:
		case LEFT_BLUE:
		case LEFT_YELLOW:
			switch (bottle[player].space[x+1][y])
			{
				// right half
				case RIGHT_RED:
					bottle[player].space[x+1][y]=SINGLE_RED;
					SetTile(first_column+x+1,BOTTLE_FIRST_ROW+y,SINGLE_RED);
					break;
				case RIGHT_BLUE:
					bottle[player].space[x+1][y]=SINGLE_BLUE;
					SetTile(first_column+x+1,BOTTLE_FIRST_ROW+y,SINGLE_BLUE);
					break;
				case RIGHT_YELLOW:
					bottle[player].space[x+1][y]=SINGLE_YELLOW;
					SetTile(first_column+x+1,BOTTLE_FIRST_ROW+y,SINGLE_YELLOW);
					break;
			}
			break;

		case RIGHT_RED:
		case RIGHT_BLUE:
		case RIGHT_YELLOW:
			switch (bottle[player].space[x-1][y])
			{
				
				case LEFT_RED:
					bottle[player].space[x-1][y]=SINGLE_RED;
					SetTile(first_column+x-1,BOTTLE_FIRST_ROW+y,SINGLE_RED);
					break;
				case LEFT_BLUE:
					bottle[player].space[x-1][y]=SINGLE_BLUE;
					SetTile(first_column+x-1,BOTTLE_FIRST_ROW+y,SINGLE_BLUE);
					break;
				case LEFT_YELLOW:
					bottle[player].space[x-1][y]=SINGLE_YELLOW;
					SetTile(first_column+x-1,BOTTLE_FIRST_ROW+y,SINGLE_YELLOW);
					break;
			}
			break;

		case BOTTOM_RED:
		case BOTTOM_BLUE:
		case BOTTOM_YELLOW:
			switch (bottle[player].space[x][y-1])
			{
				// right half
				case TOP_RED:
					bottle[player].space[x][y-1]=SINGLE_RED;
					SetTile(first_column+x,BOTTLE_FIRST_ROW+y-1,SINGLE_RED);
					break;
				case TOP_BLUE:
					bottle[player].space[x][y-1]=SINGLE_BLUE;
					SetTile(first_column+x,BOTTLE_FIRST_ROW+y-1,SINGLE_BLUE);
					break;
				case TOP_YELLOW:
					bottle[player].space[x][y-1]=SINGLE_YELLOW;
					SetTile(first_column+x,BOTTLE_FIRST_ROW+y-1,SINGLE_YELLOW);
					break;
			}
			break;

		case TOP_RED:
		case TOP_BLUE:
		case TOP_YELLOW:
			switch (bottle[player].space[x][y+1])
			{
				// right half
				case BOTTOM_RED:
					bottle[player].space[x][y+1]=SINGLE_RED;
					SetTile(first_column+x,BOTTLE_FIRST_ROW+y+1,SINGLE_RED);
					break;
				case BOTTOM_BLUE:
					bottle[player].space[x][y+1]=SINGLE_BLUE;
					SetTile(first_column+x,BOTTLE_FIRST_ROW+y+1,SINGLE_BLUE);
					break;
				case BOTTOM_YELLOW:
					bottle[player].space[x][y+1]=SINGLE_YELLOW;
					SetTile(first_column+x,BOTTLE_FIRST_ROW+y+1,SINGLE_YELLOW);
					break;
			}
			break;
	}

}

void DropFloaters(unsigned short player)
{
	bottle[player].contains_floaters=0;

	for ( y=BOTTLE_DEPTH-2 ; y < 254; y--  ) // was y> 0
	{
		for ( x=0 ; x< BOTTLE_WIDTH ; x++ )
		{
			switch (bottle[player].space[x][y])
			{

				case VIRUS_RED1 ... VIRUS_RED2:
				case VIRUS_BLUE1 ... VIRUS_BLUE2:
				case VIRUS_YELLOW1 ... VIRUS_YELLOW2:
					break;
				
				case 0:
					break;

				case SINGLE_RED:
				case SINGLE_BLUE:
				case SINGLE_YELLOW:
					// nothing under me ?
					if (  bottle[player].space[x][y+1] == 0 )
					{
						DropFilledSpace(player,x,y);
						bottle[player].contains_floaters=1;	
					}		
					break;

				case LEFT_BLUE:
				case LEFT_RED:
				case LEFT_YELLOW:
					// nothing under me or down right  ?
					if( (  bottle[player].space[x][y+1] == 0  ) &&   (   bottle[player].space[x+1][y+1] == 0  )  )
					{
							DropFilledSpace(player,x,y);
							DropFilledSpace(player,x+1,y);
							bottle[player].contains_floaters=1;
					}		
					break;
		
				case BOTTOM_BLUE:
				case BOTTOM_RED:
				case BOTTOM_YELLOW:
					
					// nothing under me ?
					if (  bottle[player].space[x][y+1] == 0 )
					{

						DropFilledSpace(player,x,y);
						DropFilledSpace(player,x,y-1);
						bottle[player].contains_floaters=1;
					}		
					break;
			}
			
		}
	}



	if ( ! bottle[player].contains_floaters )
	{	
		CheckMatching4(player);

		// send trash if needed
		if ( bottle[player].sequences_count > 1 )
		{
			QueTrash(player, bottle[player].sequences_count);
		}		

//		bottle[player].sequences_count = 0; was here
	}
		bottle[player].sequences_count = 0; //now here

}

void QueTrash(unsigned short sender, unsigned short trash_count)
{
	unsigned short receiver;

	if ( sender == PLAYER1 )
	{
		receiver = PLAYER2;
	}
	else
	{
		receiver = PLAYER1;
	}
	bottle[receiver].qued_trash += trash_count;
}

void FlingTrash(unsigned short sender)
{
	unsigned short receiver;
	unsigned short first_column; 

	if ( sender == PLAYER2 )
	{
		receiver = PLAYER2;
		first_column=BOTTLE2_FIRST_COLUMN;
	}
	else
	{
		receiver = PLAYER1;
		first_column=BOTTLE_FIRST_COLUMN;
	}


	
	z=0;

	while ( z < bottle[sender].qued_trash   )
	{
			
		x= rand() % BOTTLE_WIDTH ;
		y= 1;

		if ( !  bottle[receiver].space[x][y] ) 
		{	
			bottle[receiver].space[x][y]= SINGLE_RED + (rand() % 3)*11;

			z++;
			SetTile(first_column+x, BOTTLE_FIRST_ROW+y, bottle[receiver].space[x][y]);

		}		
	}
	bottle[sender].qued_trash = 0;
	sndFx |= BIT_4;

	bottle[receiver].contains_floaters=1;

}

void DropFilledSpace(unsigned short player, unsigned short x , unsigned short y)
{

	unsigned short first_column; 


	if ( player == PLAYER1 ) 
	{
		first_column=BOTTLE_FIRST_COLUMN;
	}
	else
	{
		first_column=BOTTLE2_FIRST_COLUMN;
	}

	// move current space 1 dowm	
	bottle[player].space[x][y+1]=bottle[player].space[x][y];
	SetTile(first_column+x,BOTTLE_FIRST_ROW+y+1,bottle[player].space[x][y+1]);

	// blank out current space
	bottle[player].space[x][y]=0;
	SetTile(first_column+x,BOTTLE_FIRST_ROW+y,0);

}


unsigned short CheckColorMatch( unsigned short tile1 , unsigned short tile2 )
{

/*
1,2,7,8,13,16,17,22 = red
3,4,9,10,14,18,19,26 = blue
5,6,11,12,15,20,21,30 = yellow
Disappear,

*/


	switch (tile1)
	{

		case 1 ... 9 :
//		case 7 ... 8 :
//		case 13:
//		case 16 ... 17:
//		case 22:
			// red 
			switch (tile2)
			{
				case 1 ... 9 :
//				case 7 ... 8 :
//				case 13:
//				case 16 ... 17:
//				case 22:
					return 1;     // RED match
			}
			break;

		case 12 ... 20 :
//		case 9 ... 10 :
//		case 14:
//		case 18 ... 19:
//		case 26:
			// blue
			switch (tile2)
			{
				case 12 ... 20 :
//				case 9 ... 10 :
//				case 14:
//				case 18 ... 19:
//				case 26:
					return 1;     // BLUE match
			}
			break;

		// must be yellow
		default :
			switch (tile2)
			{
				case 23 ... 30 :
//				case 11 ... 12 :
//				case 15:
//				case 20 ... 21:
//				case 30:
					return 1;     // YELLLOW match
			}
			break;
    }

	return 0;
}


unsigned short grab_input(unsigned short player)
{
	static bool holdKey[2];
	unsigned int c;

	c=ReadJoypad(player);

	//check if spin key was released
	if(!(c&BTN_A || c&BTN_B)) holdKey[player]=false;

	if ( auto_repeat_counter++ < 4 ) 
	{
		return 0;
	}
	auto_repeat_counter=0;

	
	if (c&BTN_A && holdKey[player]==false)
	{		
		holdKey[player]=true;
		return MOVE_SPIN_C;
		
	}
	if (c&BTN_B && holdKey[player]==false)
	{		
		holdKey[player]=true;
		return MOVE_SPIN_C_C;
		
	}

    else if ( c&BTN_RIGHT )
	{
		return MOVE_RIGHT;
	
	}
	else if ( c&BTN_LEFT )
	{

		return MOVE_LEFT;
	
	} 
	else if ( c&BTN_START )
	{

		return MOVE_START;
	
	} 
	else if ( c&BTN_DOWN )
	{
		return MOVE_DOWN;
	
	}
	
	return 0;

}


void InitBottle(unsigned short player)
{
	unsigned short virus_type;

	unsigned short column,virus_left_column;

	for ( x=0 ; x< BOTTLE_WIDTH ; x++ )
	{
		for ( y=0 ; y< BOTTLE_DEPTH ; y++ )
		{
			bottle[player].space[x][y]=0;
		}
	}

	bottle[player].check_for_sequences=0; 
    bottle[player].virus_count=0;
	bottle[player].contains_floaters=0;
	bottle[player].sequences_count=0;
	bottle[player].dissapear_anim=0;
	bottle[player].qued_trash=0;
	bottle[player].SeededNumber=StartingCounter;

	if ( player == PLAYER1 )
	{
		column=BOTTLE_FIRST_COLUMN;
		virus_left_column=VIRUS_LEFT_COLUMN1;
	}
	else
	{
		column=BOTTLE2_FIRST_COLUMN;
		virus_left_column=VIRUS_LEFT_COLUMN2;

	}

	
	// desired amount of viruses
	z = 4 * difficulty_level[player] ;
	virus_type = VIRUS_RED1+(Randomize(player) % 3 * 11);

	while ( bottle[player].virus_count < z )
	{ 
	
		x= Randomize(player) % BOTTLE_WIDTH;
		// if difficulty is greater than 15, give the viruses more possible space
		// DO NOT USE THE NUMBER 7 BELOW!!!!  IT CRASHES THE GAME!!!!!

		if (difficulty_level[player] > 15) {
			y= ( 6 + ( Randomize(player) % ( BOTTLE_DEPTH - 6) ) ) ;
		}
		else{
			y= ( 8 + ( Randomize(player) % ( BOTTLE_DEPTH - 8) ) ) ;
		}
		

		if ( !  bottle[player].space[x][y]) 
		{	
			if (CheckColorMatch (virus_type, bottle[player].space[x-1][y]) && CheckColorMatch (virus_type, bottle[player].space[x-2][y]) )
			{}
			else if (CheckColorMatch (virus_type, bottle[player].space[x][y-1]) && CheckColorMatch (virus_type, bottle[player].space[x][y-2]))
			{}
			else if (CheckColorMatch (virus_type, bottle[player].space[x+1][y]) && CheckColorMatch (virus_type, bottle[player].space[x+2][y]))
			{}
			else if (CheckColorMatch (virus_type, bottle[player].space[x][y+1]) && CheckColorMatch (virus_type, bottle[player].space[x][y+2]))
			{}
			else if (CheckColorMatch (virus_type, bottle[player].space[x+1][y]) && CheckColorMatch (virus_type, bottle[player].space[x-1][y]))
			{}
			else if (CheckColorMatch (virus_type, bottle[player].space[x][y+1]) && CheckColorMatch (virus_type, bottle[player].space[x][y-1]))
			{}
			else 
			{
				bottle[player].space[x][y]= virus_type;
			
				bottle[player].virus_count++;
				SetTile(column+x, BOTTLE_FIRST_ROW+y, bottle[player].space[x][y]);
			}

		}
		virus_type+=11;
		if (virus_type > VIRUS_YELLOW2) virus_type = VIRUS_RED1;

	}



	PrintByte(virus_left_column, VIRUS_LEFT_ROW, bottle[player].virus_count,false);

}




void AnimateVirus()
{

	if ( virus_anim_counter < 15 )
	{
		 virus_anim_counter++;
	}
	else
	{
		virus_anim_counter=0;

		for ( x=0 ; x< BOTTLE_WIDTH ; x++ )
		{
			for ( y=0 ; y< BOTTLE_DEPTH ; y++ )
			{
				switch ( bottle[PLAYER1].space[x][y] )
				{
					case VIRUS_RED1:
					case VIRUS_BLUE1:
					case VIRUS_YELLOW1:

						bottle[PLAYER1].space[x][y]++;
						SetTile(BOTTLE_FIRST_COLUMN + x,BOTTLE_FIRST_ROW  + y ,bottle[PLAYER1].space[x][y]);
						SetTile(BOTTLE_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,bottle[PLAYER1].space[x][y]);
						break;

					case VIRUS_RED2:
					case VIRUS_BLUE2:
					case VIRUS_YELLOW2:

						bottle[PLAYER1].space[x][y]--;
						SetTile(BOTTLE_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,bottle[PLAYER1].space[x][y]);
						SetTile(BOTTLE_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,bottle[PLAYER1].space[x][y]);
						break;

				}

				switch ( bottle[PLAYER2].space[x][y] )
				{
					case VIRUS_RED1:
					case VIRUS_BLUE1:
					case VIRUS_YELLOW1:

						bottle[PLAYER2].space[x][y]++;
						SetTile(BOTTLE2_FIRST_COLUMN + x,BOTTLE_FIRST_ROW  + y ,bottle[PLAYER2].space[x][y]);
						SetTile(BOTTLE2_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,bottle[PLAYER2].space[x][y]);
						break;

					case VIRUS_RED2:
					case VIRUS_BLUE2:
					case VIRUS_YELLOW2:

						bottle[PLAYER2].space[x][y]--;
						SetTile(BOTTLE2_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,bottle[PLAYER2].space[x][y]);
						SetTile(BOTTLE2_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,bottle[PLAYER2].space[x][y]);
						break;

				}

			}
		}

		
	}
}


void NewHiScore(){
	StopSong(song_drmario_main);
	TriggerFx(19,0xc0,true);
	StartSong(Victory);

	StartingCounter = 1;
	unsigned char i = 0, j = 0, z=0;
	char NewName[6] = "AAAAAA";

	if ((difficulty_level[PLAYER1] == 21) && (round_time/60 < HiScoreFour)){
		HiScoreFour = round_time/60;
		for(i=0; i<6; i++){
		NameFour[i] = NewName[i];
		j = 8;
		}
	}
	else if (difficulty_level[PLAYER1] == 16 && (round_time/60 < HiScoreOne)){
		HiScoreOne = round_time/60;
		for(i=0; i<6; i++){
		NameOne[i] = NewName[i];
		j = 13;
		}
	}
	else if (difficulty_level[PLAYER1] == 11 && (round_time/60 < HiScoreTwo)){
		HiScoreTwo = round_time/60;
		for(i=0; i<6; i++){
		NameTwo[i] = NewName[i];
		j = 18;
		}
	}
	else{
		HiScoreThree = round_time/60;
		for(i=0; i<6; i++){
		NameThree[i] = NewName[i];
		j = 23;
		}
	}

	PrintHiScores();

	while (StartingCounter < 30000){
		if(ReadJoypad(0)&BTN_UP)
		{
			while(ReadJoypad(0)!=0); //wait for key release
			NewName[z]++;
			if(NewName[z] > 93) NewName[z] = 32;
			StartingCounter = 1;
		}
		if(ReadJoypad(0)&BTN_DOWN)
		{
			while(ReadJoypad(0)!=0); //wait for key release
			NewName[z]--;
			if(NewName[z] < 32) NewName[z] = 93;
			StartingCounter = 1;
		}
		if(ReadJoypad(0)&BTN_LEFT)
		{
			while(ReadJoypad(0)!=0); //wait for key release
			if (z > 0) z--;
			StartingCounter = 500;
		}
		if(ReadJoypad(0)&BTN_RIGHT)
		{
			while(ReadJoypad(0)!=0); //wait for key release
			if (z < 5) z++;
			StartingCounter = 500;
		}
		if(ReadJoypad(0)&BTN_START)
		{
			StartingCounter = 30000;
		}

		CheckInsertCoin(false);
		for(unsigned char i=0; i<6;i++){
			if (i == z && (StartingCounter % 1000) > 500) SetTile(29+z,j,0); //blink cursor
			else PrintChar(29+i, j, NewName[i]);
		}
		StartingCounter++; // Increment counter for time out and flashing curser.
	}
	
	PrintChar(29+z, j, NewName[z]); // In case we hit the A button when the cursor is on.
	
	if (j == 13)
		for(i=0; i<6; i++) NameOne[i] = NewName[i];
	else if (j==18)
		for(i=0; i<6; i++) NameTwo[i] = NewName[i];
	else if (j==23)
		for(i=0; i<6; i++) NameThree[i] = NewName[i];
	else 
		for(i=0; i<6; i++) NameFour[i] = NewName[i];

	SaveHighScore();

}

void PrintHiScores(){
	for(unsigned char i=0; i<6;i++){
		PrintChar(29+i, 13, NameOne[i]);
		}
	PrintByte(30, 14, HiScoreOne/60,false);
	Print(31, 14, strM);
	PrintByte(34, 14, HiScoreOne%60,false);
	Print(35, 14, strS);

	for(unsigned char i=0; i<6;i++){
		PrintChar(29+i, 18, NameTwo[i]);
		}
	PrintByte(31, 19, HiScoreTwo/60,false);
	Print(32, 19, strM);
	PrintByte(35, 19, HiScoreTwo%60,false);
	Print(36, 19, strS);
	for(unsigned char i=0; i<6;i++){
		PrintChar(29+i, 23, NameThree[i]);
		}
	PrintByte(31, 24, HiScoreThree/60,false);
	Print(32, 24, strM);
	PrintByte(35, 24, HiScoreThree%60,false);
	Print(36, 24, strS);

	for(unsigned char i=0; i<6;i++){
		PrintChar(29+i, 8, NameFour[i]);
		}
	PrintByte(31, 9, HiScoreFour/60,false);
	Print(32, 9, strM);
	PrintByte(35, 9, HiScoreFour%60,false);
	Print(36, 9, strS);


	Print(29,21,strStudent);
	Print(29,16,strIntern);
	Print(29,11,strDoctor);
	Print(29,6,strSurgeon);


	Print(28,4,strBesttime);


}


/**
        Loads hi score from eeprom (requires a formatted and non-full eeprom).
*/

unsigned short PowFunc4(unsigned short num) {
	if (num == 1) return 4;
	else if (num == 2) return 16;
	else if (num == 3) return 64;
	else if (num == 4) return 256;
	else if (num == 5) return 1024;
	else return 1;
}

unsigned short PowFunc2(unsigned short num) {
	if (num == 1) return 2;
	else if (num == 2) return 4;
	else if (num == 3) return 8;
	else if (num == 4) return 16;
	else if (num == 5) return 32;
	else return 1;
}


void LoadHighScore() {
        struct EepromBlockStruct ebs;

        ebs.id = DROPPER_ID;

        if (!isEepromFormatted() || EepromReadBlock(DROPPER_ID, &ebs))
                return;

//		HiScoreOne = ebs.data[2]*256 +ebs.data[3];	
//		HiScoreTwo = ebs.data[4]*256 +ebs.data[5];
//		HiScoreThree = ebs.data[6]*256 +ebs.data[7];

		HiScoreOne = 0;	
		HiScoreTwo = 0;
		HiScoreThree = 0;
		HiScoreFour = 0;


		for(unsigned char i = 8; i<14; i++)
		{
			NameOne[i-8] = ebs.data[i]%64 + 32;
			NameTwo[i-8] = ebs.data[i+6]%64  + 32;
			NameThree[i-8] = ebs.data[i+12]%64  + 32;
			NameFour[i-8] = ebs.data[i-6]%64 + 32;
			HiScoreOne += (ebs.data[i]/64) * PowFunc4(i-8);	
			HiScoreTwo += (ebs.data[i+6]/64) * PowFunc4(i-8);	
			HiScoreThree += (ebs.data[i+12]/64) * PowFunc4(i-8);	
			HiScoreFour += (ebs.data[i-6]/64) * PowFunc4(i-8);	
		}

		PLevel = ebs.data[0];

		coinsPerCreditOne = ebs.data[1] % BIT_4;
		coinsPerCreditTwo = ebs.data[1] / BIT_5;
}

/*
        Saves the current hi score to eeprom (requires a formatted and non-full eeprom).
*/
void SaveHighScore() {
        struct EepromBlockStruct ebs;

        ebs.id = DROPPER_ID;

        if (!isEepromFormatted())
                return;

	ebs.data[0] = PLevel;
	ebs.data[1] = coinsPerCreditOne+16*coinsPerCreditTwo;
	ebs.data[2] = HiScoreOne / BIT_9;
	ebs.data[3] = HiScoreOne % BIT_9;
	ebs.data[4] = HiScoreTwo / BIT_9;
	ebs.data[5] = HiScoreTwo % BIT_9;
	ebs.data[6] = HiScoreThree / BIT_9;
	ebs.data[7] = HiScoreThree % BIT_9;
	
	for(unsigned char i = 8; i<14; i++)
	{
		ebs.data[i] =  (NameOne[i-8] -32) + (64 * ((HiScoreOne % PowFunc4(i-7)) / PowFunc4(i-8)));
		ebs.data[i+6] = (NameTwo[i-8] -32) + (64 * ((HiScoreTwo % PowFunc4(i-7)) / PowFunc4(i-8)));
		ebs.data[i+12] = (NameThree[i-8]-32) + (64 * ((HiScoreThree % PowFunc4(i-7)) / PowFunc4(i-8)));
		ebs.data[i-6] = (NameFour[i-8]-32) + (64 * ((HiScoreFour % PowFunc4(i-7)) / PowFunc4(i-8)));
	}

	//Write to EEPROM
	EepromWriteBlock(&ebs);

}

void SinglePlayerMenu(){
	char c = 0;

	//DrawMap(5,15,drmario_map_single_menu);
	Print(12,15,strStudent);
	Print(12,17,strIntern);
	Print(12,19,strDoctor);
	Print(12,21,strSurgeon);

	SetTile(10,15+2*c,CURSOR_TILE); //draw cursor


	while(1){
	
		if(ReadJoypad(0)&(BTN_START|BTN_A|BTN_B)){
			
			while(ReadJoypad(0)!=0); //wait for key release
			difficulty_level[PLAYER1] = c * 5 + 6;
			return; 
		}

		if((ReadJoypad(0)&BTN_UP) || (ReadJoypad(0)&BTN_DOWN)){
			SetTile(10,15+c*2,0); //erase old cursor
			sndFx |= BIT_1;

			if(ReadJoypad(0)&BTN_UP){
				if(c<1) c=0;
				else c--;
				

			}else if(ReadJoypad(0)&BTN_DOWN){
				if(c>2)	c=3;
				else c++;
			}
			
		SetTile(10,15+c*2,CURSOR_TILE); //draw new cursor
		while(ReadJoypad(0)!=0); //wait for key release
		}
	}
}

void OptionsMenu(){
	char c = 0;
	ClearVram();
	
	Print(14,8,strGameOptions);
	
	Print(11,13,strCoinsPerCreditOne);
	PrintByte(28,13,coinsPerCreditOne,false);
	Print(11,14,strCoinsPerCreditTwo);
	PrintByte(28,14,coinsPerCreditTwo,false);
	
	Print(11,15,strPLevel);
	PrintByte(28,15,PLevel,true);

	Print(11,16,strBackToMenu);

	Print(6,21,strResetHighScore);
	Print(5,22,strResetHighScoreTwo);

	SetTile(10,13+c,CURSOR_TILE); //draw cursor

	while(1){
		if(ReadJoypad(0)&BTN_SR) SoftReset();

	
		if(ReadJoypad(0)&(BTN_START|BTN_A|BTN_B) && c == 3){
			
			while(ReadJoypad(0)!=0); //wait for key release
			SaveHighScore();
			DrawMainMenu();
			return;
		}

		if(ReadJoypad(0)&BTN_LEFT){
			if(c==0 && (coinsPerCreditOne) > 0){
				coinsPerCreditOne--;
				PrintByte(28,13,coinsPerCreditOne,false);
				sndFx |= BIT_1;
			}
			else if(c==1 && (coinsPerCreditTwo) > 0){
				coinsPerCreditTwo--;
				PrintByte(28,14,coinsPerCreditTwo,false);
				sndFx |= BIT_1;
			}
			else if(c==2 && PLevel>1){
				PLevel--;
				PrintByte(28,15,PLevel,true);
				sndFx |= BIT_1;
			}
			while(ReadJoypad(0)!=0); //wait for key release	
		}
		if(ReadJoypad(0)&BTN_RIGHT){
			if(c==0 && coinsPerCreditOne < 4){
				coinsPerCreditOne++;
				PrintByte(28,13,coinsPerCreditOne,false);
				sndFx |= BIT_1;
			}
			else if(c==1 && coinsPerCreditTwo < 4){
				coinsPerCreditTwo++;
				PrintByte(28,14,coinsPerCreditTwo,false);
				sndFx |= BIT_1;
			}	
			else if(c==2 && PLevel<5){
				PLevel++;
				PrintByte(28,15,PLevel,true);
				sndFx |= BIT_1;
			}
			while(ReadJoypad(0)!=0); //wait for key release	
		}
		if((ReadJoypad(0)&BTN_UP) || (ReadJoypad(0)&BTN_DOWN)){
			SetTile(10,13+c,0); //erase old cursor
			sndFx |= BIT_1;

			if(ReadJoypad(0)&BTN_UP){
				if(c<1) c=3;
				else c--;
				

			}else if(ReadJoypad(0)&BTN_DOWN){
				if(c>2)	c=0;
				else c++;
			}
			
		SetTile(10,13+c,CURSOR_TILE); //draw new cursor
		while(ReadJoypad(0)!=0); //wait for key release
		}
	}
}

void playFx(){

	if (sndFx & BIT_5) TriggerFx(32,0xff,true);
	if (sndFx & BIT_4) TriggerFx(22,0xff,true);
	if (sndFx & BIT_3) TriggerFx(19,0xc0,true);
	if (sndFx & BIT_2) TriggerFx(18,0xa0,true);
	if (sndFx & BIT_1) TriggerFx(1,0xff,true);

	sndFx = 0;
}
